;;; guile-user.scm

;;	Copyright (C) 1995,1996,1997,1998,1999,2000,2002 Free Software Foundation, Inc.
;;
;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this software; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
;; Boston, MA 02111-1307 USA
;;
;; As a special exception, the Free Software Foundation gives permission
;; for additional uses of the text contained in its release of GUILE.
;;
;; The exception is that, if you link the GUILE library with other files
;; to produce an executable, this does not by itself cause the
;; resulting executable to be covered by the GNU General Public License.
;; Your use of that executable is in no way restricted on account of
;; linking the GUILE library code into it.
;;
;; This exception does not however invalidate any other reasons why
;; the executable file might be covered by the GNU General Public License.
;;
;; This exception applies only to the code released by the
;; Free Software Foundation under the name GUILE.  If you copy
;; code from other Free Software Foundation releases into a copy of
;; GUILE, as the General Public License permits, the exception does
;; not apply to the code that you add in this way.  To avoid misleading
;; anyone as to the status of such modified files, you must delete
;; this exception notice from them.
;;
;; If you write modifications of your own for GUILE, it is your choice
;; whether to permit this exception to apply to your modifications.
;; If you do not wish that, delete this exception notice.

;;; Use :autoload to minimize startup time.

(define-module (guile-user)
  :use-module (guile)             ; so that bindings will be checked here first
  :autoload (ice-9 session) (help apropos apropos-internal
                                  apropos-fold apropos-fold-accessible
                                  apropos-fold-exported apropos-fold-all
                                  source arity system-module)
  :autoload (ice-9 debug) (frame-number->index trace untrace)
  :autoload (ice-9 debugger) (debug))	; load debugger on demand

(if (memq 'threads *features*)
    (define-module (guile-user)
      :autoload (ice-9 threads) (%thread-handler
                                 make-thread begin-thread
                                 with-mutex monitor)))

(if (memq 'regex *features*)
    (define-module (guile-user)
      :autoload (ice-9 regex) (match:count
                               match:string match:prefix match:suffix
                               regexp-match? regexp-quote
                               match:start match:end match:substring
                               string-match regexp-substitute fold-matches
                               list-matches regexp-substitute/global)))

(append! %load-path
         ;; TODO: Move this to libguile/load.c eventually.
         (let ((gbi (lambda (key) (assq-ref %guile-build-info key))))
           (list (format #f "~A/~A" (gbi 'pkglibdir) (gbi 'guileversion)))))

(define-module (guile-user)
  ;; Note that this is also done for module `(guile)'; there is
  ;; currently no way to construct an "exporting autoload".
  :autoload (modules rgx) (regcomp
                           regexec compiled-regexp?
                           regexp->dfa dfa-fork reset-dfa!
                           dfa-final-tag dfa-continuable? advance-dfa!))

;;; guile-user.scm ends here
