;;; networking.scm

;;	Copyright (C) 1995,1996,1997,1998,1999,2000,2002 Free Software Foundation, Inc.
;;
;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this software; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
;; Boston, MA 02111-1307 USA
;;
;; As a special exception, the Free Software Foundation gives permission
;; for additional uses of the text contained in its release of GUILE.
;;
;; The exception is that, if you link the GUILE library with other files
;; to produce an executable, this does not by itself cause the
;; resulting executable to be covered by the GNU General Public License.
;; Your use of that executable is in no way restricted on account of
;; linking the GUILE library code into it.
;;
;; This exception does not however invalidate any other reasons why
;; the executable file might be covered by the GNU General Public License.
;;
;; This exception applies only to the code released by the
;; Free Software Foundation under the name GUILE.  If you copy
;; code from other Free Software Foundation releases into a copy of
;; GUILE, as the General Public License permits, the exception does
;; not apply to the code that you add in this way.  To avoid misleading
;; anyone as to the status of such modified files, you must delete
;; this exception notice from them.
;;
;; If you write modifications of your own for GUILE, it is your choice
;; whether to permit this exception to apply to your modifications.
;; If you do not wish that, delete this exception notice.

;;; installed-scm-file

;;;; Copyright (C) 1999 Free Software Foundation, Inc.
;;;;
;;;; This program is free software; you can redistribute it and/or modify
;;;; it under the terms of the GNU General Public License as published by
;;;; the Free Software Foundation; either version 2, or (at your option)
;;;; any later version.
;;;;
;;;; This program is distributed in the hope that it will be useful,
;;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;;; GNU General Public License for more details.
;;;;
;;;; You should have received a copy of the GNU General Public License
;;;; along with this software; see the file COPYING.  If not, write to
;;;; the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
;;;; Boston, MA 02111-1307 USA
;;;;

(define (gethostbyaddr addr) (gethost addr))
(define (gethostbyname name) (gethost name))

(define (getnetbyaddr addr) (getnet addr))
(define (getnetbyname name) (getnet name))

(define (getprotobyname name) (getproto name))
(define (getprotobynumber addr) (getproto addr))

(define (getservbyname name proto) (getserv name proto))
(define (getservbyport port proto) (getserv port proto))

;; Initialize an internal stream from which host objects may be read.  This
;; procedure must be called before any calls to @code{gethostent}, and may
;; also be called afterward to reset the host entry stream.  If
;; @var{stayopen} is supplied and is not @code{#f}, the database is not
;; closed by subsequent @code{gethostbyname} or @code{gethostbyaddr} calls,
;; possibly giving an efficiency gain.
(define (sethostent . stayopen)
  (if (pair? stayopen)
      (sethost (car stayopen))
      (sethost #f)))
;; Initialize an internal stream from which network objects may be read.  This
;; procedure must be called before any calls to @code{getnetent}, and may
;; also be called afterward to reset the net entry stream.  If
;; @var{stayopen} is supplied and is not @code{#f}, the database is not
;; closed by subsequent @code{getnetbyname} or @code{getnetbyaddr} calls,
;; possibly giving an efficiency gain.
(define (setnetent . stayopen)
  (if (pair? stayopen)
      (setnet (car stayopen))
      (setnet #f)))
;; Initialize an internal stream from which protocol objects may be read.  This
;; procedure must be called before any calls to @code{getprotoent}, and may
;; also be called afterward to reset the protocol entry stream.  If
;; @var{stayopen} is supplied and is not @code{#f}, the database is not
;; closed by subsequent @code{getprotobyname} or @code{getprotobynumber} calls,
;; possibly giving an efficiency gain.
(define (setprotoent . stayopen)
  (if (pair? stayopen)
      (setproto (car stayopen))
      (setproto #f)))
;; Initialize an internal stream from which service objects may be read.  This
;; procedure must be called before any calls to @code{getservent}, and may
;; also be called afterward to reset the service entry stream.  If
;; @var{stayopen} is supplied and is not @code{#f}, the database is not
;; closed by subsequent @code{getservbyname} or @code{getservbyport} calls,
;; possibly giving an efficiency gain.
(define (setservent . stayopen)
  (if (pair? stayopen)
      (setserv (car stayopen))
      (setserv #f)))

;; Return the next host object from the host database, or @code{#f} if
;; there are no more hosts to be found (or an error has been encountered).
;; This procedure may not be used before @code{sethostent} has been called.
(define (gethostent) (gethost))
;; Return the next entry from the network database.
(define (getnetent) (getnet))
;; Return the next entry from the protocol database.
(define (getprotoent) (getproto))
;; Return the next entry from the services database.
(define (getservent) (getserv))

;; Close the stream used by @code{gethostent}.  The return value is unspecified.
(define (endhostent) (sethost))
;; Close the stream used by @code{getnetent}.  The return value is unspecified.
(define (endnetent) (setnet))
;; Close the stream used by @code{getprotoent}.  The return value is unspecified.
(define (endprotoent) (setproto))
;; Close the stream used by @code{getservent}.  The return value is unspecified.
(define (endservent) (setserv))

;; The "official" hostname for @var{host}.
(define (hostent:name obj) (vector-ref obj 0))
;; A list of aliases for @var{host}.
(define (hostent:aliases obj) (vector-ref obj 1))
;; The host address type.  For hosts with Internet addresses, this will
;; return @code{AF_INET}.
(define (hostent:addrtype obj) (vector-ref obj 2))
;; The length of each address for @var{host}, in bytes.
(define (hostent:length obj) (vector-ref obj 3))
;; The list of network addresses associated with @var{host}.
(define (hostent:addr-list obj) (vector-ref obj 4))

;; The "official" network name.
(define (netent:name obj) (vector-ref obj 0))
;; A list of aliases for the network.
(define (netent:aliases obj) (vector-ref obj 1))
;; The type of the network number.  Currently, this returns only
;; @code{AF_INET}.
(define (netent:addrtype obj) (vector-ref obj 2))
;; The network number.
(define (netent:net obj) (vector-ref obj 3))

;; The "official" protocol name.
(define (protoent:name obj) (vector-ref obj 0))
;; A list of aliases for the protocol.
(define (protoent:aliases obj) (vector-ref obj 1))
;; The protocol number.
(define (protoent:proto obj) (vector-ref obj 2))

;; The "official" name of the network service.
(define (servent:name obj) (vector-ref obj 0))
;; A list of aliases for the network service.
(define (servent:aliases obj) (vector-ref obj 1))
;; The Internet port used by the service.
(define (servent:port obj) (vector-ref obj 2))
;; The protocol used by the service.  A service may be listed many times
;; in the database under different protocol names.
(define (servent:proto obj) (vector-ref obj 3))

(define (sockaddr:fam obj) (vector-ref obj 0))
(define (sockaddr:path obj) (vector-ref obj 1))
(define (sockaddr:addr obj) (vector-ref obj 1))
(define (sockaddr:port obj) (vector-ref obj 2))

;;; networking.scm ends here
