;;; properties.scm

;;	Copyright (C) 1995,1996,1997,1998,1999,2000,2002 Free Software Foundation, Inc.
;;
;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this software; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
;; Boston, MA 02111-1307 USA
;;
;; As a special exception, the Free Software Foundation gives permission
;; for additional uses of the text contained in its release of GUILE.
;;
;; The exception is that, if you link the GUILE library with other files
;; to produce an executable, this does not by itself cause the
;; resulting executable to be covered by the GNU General Public License.
;; Your use of that executable is in no way restricted on account of
;; linking the GUILE library code into it.
;;
;; This exception does not however invalidate any other reasons why
;; the executable file might be covered by the GNU General Public License.
;;
;; This exception applies only to the code released by the
;; Free Software Foundation under the name GUILE.  If you copy
;; code from other Free Software Foundation releases into a copy of
;; GUILE, as the General Public License permits, the exception does
;; not apply to the code that you add in this way.  To avoid misleading
;; anyone as to the status of such modified files, you must delete
;; this exception notice from them.
;;
;; If you write modifications of your own for GUILE, it is your choice
;; whether to permit this exception to apply to your modifications.
;; If you do not wish that, delete this exception notice.

;;; {Symbol Properties}
;;;

;; From @var{sym}'s property list, return the value for property
;; @var{prop}.  The assumption is that @var{sym}'s property list is an
;; association list whose keys are distinguished from each other using
;; @code{equal?}; @var{prop} should be one of the keys in that list.  If
;; the property list has no entry for @var{prop}, @code{symbol-property}
;; returns @code{#f}.
(define (symbol-property sym prop)
  (let ((pair (assoc prop (symbol-pref sym))))
    (and pair (cdr pair))))

;; In @var{sym}'s property list, set the value for property @var{prop} to
;; @var{val}, or add a new entry for @var{prop}, with value @var{val}, if
;; none already exists.  For the structure of the property list, see
;; @code{symbol-property}.
(define (set-symbol-property! sym prop val)
  (let ((pair (assoc prop (symbol-pref sym))))
    (if pair
	(set-cdr! pair val)
	(symbol-pset! sym (acons prop val (symbol-pref sym))))))

;; From @var{sym}'s property list, remove the entry for property
;; @var{prop}, if there is one.  For the structure of the property list,
;; see @code{symbol-property}.
(define (symbol-property-remove! sym prop)
  (let ((pair (assoc prop (symbol-pref sym))))
    (if pair
	(symbol-pset! sym (delq! pair (symbol-pref sym))))))

;;; {General Properties}
;;;

(define (make-object-property)
  (let ((prop (gensym "%%object-property-")))
    (make-procedure-with-setter
     (lambda (obj) (object-property obj prop))
     (lambda (obj val) (set-object-property! obj prop val)))))

;;; properties.scm ends here
