;;; runtime-options.scm

;;	Copyright (C) 1995,1996,1997,1998,1999,2000,2002 Free Software Foundation, Inc.
;;
;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this software; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
;; Boston, MA 02111-1307 USA
;;
;; As a special exception, the Free Software Foundation gives permission
;; for additional uses of the text contained in its release of GUILE.
;;
;; The exception is that, if you link the GUILE library with other files
;; to produce an executable, this does not by itself cause the
;; resulting executable to be covered by the GNU General Public License.
;; Your use of that executable is in no way restricted on account of
;; linking the GUILE library code into it.
;;
;; This exception does not however invalidate any other reasons why
;; the executable file might be covered by the GNU General Public License.
;;
;; This exception applies only to the code released by the
;; Free Software Foundation under the name GUILE.  If you copy
;; code from other Free Software Foundation releases into a copy of
;; GUILE, as the General Public License permits, the exception does
;; not apply to the code that you add in this way.  To avoid misleading
;; anyone as to the status of such modified files, you must delete
;; this exception notice from them.
;;
;; If you write modifications of your own for GUILE, it is your choice
;; whether to permit this exception to apply to your modifications.
;; If you do not wish that, delete this exception notice.

;;; {Run-time options}

(define %define-option-interface
  (let* ((option-name car)
	 (option-value cadr)
	 (option-documentation caddr)

	 (print-option (lambda (option)
			 (display (option-name option))
			 (if (< (string-length
				 (symbol->string (option-name option)))
				8)
			     (display #\tab))
			 (display #\tab)
			 (display (option-value option))
			 (display #\tab)
			 (display (option-documentation option))
			 (newline)))

	 ;; Below follow the macros defining the run-time option interfaces.

	 (make-options (lambda (interface)
			 `(lambda args
			    (cond ((null? args) (,interface))
				  ((list? (car args))
				   (,interface (car args)) (,interface))
				  (else (for-each ,print-option
						  (,interface #t)))))))

	 (make-enable (lambda (interface)
			`(lambda flags
			   (,interface (append flags (,interface)))
			   (,interface))))

	 (make-disable (lambda (interface)
			 `(lambda flags
			    (let ((options (,interface)))
			      (for-each (lambda (flag)
					  (set! options (delq! flag options)))
					flags)
			      (,interface options)
			      (,interface)))))

	 (make-set! (lambda (interface)
		      `((name exp)
			(,'quasiquote
			 (begin (,interface (append (,interface)
						    (list '(,'unquote name)
							  (,'unquote exp))))
				(,interface)))))))
    (procedure->memoizing-macro
     (lambda (exp env)
       (cons 'begin
	     (let* ((option-group (cadr exp))
		    (interface (car option-group)))
	       (append (map (lambda (name constructor)
			      `(define ,name
				 ,(constructor interface)))
			    (cadr option-group)
			    (list make-options
				  make-enable
				  make-disable))
		       (map (lambda (name constructor)
			      `(defmacro ,name
				 ,@(constructor interface)))
			    (caddr option-group)
			    (list make-set!)))))))))

(%define-option-interface
 (eval-options-interface
  (eval-options eval-enable eval-disable)
  (eval-set!)))

(%define-option-interface
 (debug-options-interface
  (debug-options debug-enable debug-disable)
  (debug-set!)))

(%define-option-interface
 (evaluator-traps-interface
  (traps trap-enable trap-disable)
  (trap-set!)))

(%define-option-interface
 (read-options-interface
  (read-options read-enable read-disable)
  (read-set!)))

(%define-option-interface
 (print-options-interface
  (print-options print-enable print-disable)
  (print-set!)))

;;; runtime-options.scm ends here
