;;; debug-common.scm

;;	Copyright (C) 2002 Free Software Foundation, Inc.
;;
;; This file is part of GUILE
;;
;; GUILE is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.
;;
;; GUILE is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this software; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
;; Boston, MA 02111-1307 USA
;;
;; As a special exception, the Free Software Foundation gives permission
;; for additional uses of the text contained in its release of GUILE.
;;
;; The exception is that, if you link the GUILE library with other files
;; to produce an executable, this does not by itself cause the
;; resulting executable to be covered by the GNU General Public License.
;; Your use of that executable is in no way restricted on account of
;; linking the GUILE library code into it.
;;
;; This exception does not however invalidate any other reasons why
;; the executable file might be covered by the GNU General Public License.
;;
;; This exception applies only to the code released by the
;; Free Software Foundation under the name GUILE.  If you copy
;; code from other Free Software Foundation releases into a copy of
;; GUILE, as the General Public License permits, the exception does
;; not apply to the code that you add in this way.  To avoid misleading
;; anyone as to the status of such modified files, you must delete
;; this exception notice from them.
;;
;; If you write modifications of your own for GUILE, it is your choice
;; whether to permit this exception to apply to your modifications.
;; If you do not wish that, delete this exception notice.

;;; Author: Thien-Thi Nguyen <ttn@gnu.org>

;;; Commentary:

;; This module contains debugging support for the (lang foo-debug) modules.

;;; Code:

(define-module (lang debug-common)
  :autoload (lang grammar) (production-symbol production-expansion)
  :export (;; pretty printers
           pp-lexer-spec
           pp-grammar
           ;; nicknames
           nick
           nick!))

(define (pp-lexer-spec spec)
  (for-each (lambda (item)
	      (display (car item))
	      (display #\tab) (display #\tab)
	      (cond
	       ((symbol? (cadr item))	(display (cadr item)))
	       ((not (cadr item))	(display "ignored"))
	       (else			(display "!!! special")))
	      (display #\space)
	      (if (memq :shortest item)
		  (display :shortest))
	      (newline))
	    spec))

(define (pp-grammar spec)
  (let loop ((current-sym #f)
	     (pos spec))
    (or (null? pos)
        (let ((prod (car pos)))
          (if (eq? current-sym (production-symbol prod))
              (begin
                (display (make-string (string-length current-sym) #\space))
                (display "  | "))
              (begin
                (newline)
                (display (production-symbol prod))
                (display " -> ")))
          (for-each (lambda (elt) (display elt) (display " "))
                    (production-expansion prod))
          (cond ((object-property prod 'nick)
                 => (lambda (nick)
                      (display #\tab) (display #\tab)
                      (display nick))))
          (newline)
          (loop (production-symbol prod) (cdr pos))))))

;; hash support

(define (hex-addr obj)
  (number->string (object-address obj) 16))

(define (nick obj)
  (and obj (or (object-property obj 'nick)
               (and (list? obj)
                    (null? obj)
                    obj)
               (hex-addr obj))))

(define (nick! obj nick)
  (set-object-property! obj 'nick nick)
  obj)

;;; debug-common.scm ends here
