;;; lr1-debug.scm

;;	Copyright (C) 2002 Free Software Foundation, Inc.
;;
;; This file is part of GUILE
;;
;; GUILE is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.
;;
;; GUILE is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this software; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
;; Boston, MA 02111-1307 USA
;;
;; As a special exception, the Free Software Foundation gives permission
;; for additional uses of the text contained in its release of GUILE.
;;
;; The exception is that, if you link the GUILE library with other files
;; to produce an executable, this does not by itself cause the
;; resulting executable to be covered by the GNU General Public License.
;; Your use of that executable is in no way restricted on account of
;; linking the GUILE library code into it.
;;
;; This exception does not however invalidate any other reasons why
;; the executable file might be covered by the GNU General Public License.
;;
;; This exception applies only to the code released by the
;; Free Software Foundation under the name GUILE.  If you copy
;; code from other Free Software Foundation releases into a copy of
;; GUILE, as the General Public License permits, the exception does
;; not apply to the code that you add in this way.  To avoid misleading
;; anyone as to the status of such modified files, you must delete
;; this exception notice from them.
;;
;; If you write modifications of your own for GUILE, it is your choice
;; whether to permit this exception to apply to your modifications.
;; If you do not wish that, delete this exception notice.

;;; Author: Thien-Thi Nguyen <ttn@gnu.org>

;;; Commentary:

;; This module is a drop-in replacement for module (lang lr1) with some
;; additional debugging features.
;;
;; TODO: Document better.

;;; Code:

(define-module (lang lr1-debug)
  :use-module (ice-9 common-list)
  :use-module (lang debug-common)
  :use-module (lang grammar)
  :use-module ((lang lr1) :renamer (symbol-prefix-proc 'ACTUAL:))
  :export (;; mimic exports from module (lang lr1)
           parser-value-cons
           parser-value-car
           parser-value-cdr
           parser-value-cdr-ref
           parser-context-cons
           parser-context-car
           parser-context-cdr
           parser-context-cdr-ref
           make-parser-state
           parser-state-item-set
           parser-start-state
           parser-action
           lr1-parse
           ;; "native" (defined here)
           VALIDATE:state-stack-entry
           display-state-stack-entry))

;; intimacy w/ module (lang lr0)

(define *lr0* (resolve-module '(lang lr0)))

(defmacro intimate-lr0 (name)
  `(define ,name (module-ref *lr0* ',name)))

(intimate-lr0 item-index)

;; intimacy w/ module (lang lr1)

(define *lr1* (resolve-module '(lang lr1)))

(defmacro intimate-lr1 (name)
  `(define ,name (module-ref *lr1* ',name)))

(intimate-lr1 parser-state-action-cache)

;; type invariance validation

(define (VALIDATE:state-stack-entry obj)
  obj)

;; display

(define (display-state-stack-entry state)
  (format #t "~A | ~A\n"
          (nick state)
          (map car (parser-state-action-cache state))))

;; poke debugging vars

(module-define! *lr1* 'parser-context-cons-add-nickname
                (lambda (state)
                  (nick! state
                         (format #f "~A"
                                 (map item-index
                                      (parser-state-item-set state))))
                  state))

(module-define! *lr1* 'parser-loop-pk
                (lambda (state-stack token-type action-type)
                  (format #t "parser-loop: ~A ~A\n" action-type token-type)
                  (for-each display-state-stack-entry state-stack)
                  (newline)))

;; wrap the actual

(define parser-value-cons       ACTUAL:parser-value-cons)
(define parser-value-car        ACTUAL:parser-value-car)
(define parser-value-cdr        ACTUAL:parser-value-cdr)
(define parser-value-cdr-ref    ACTUAL:parser-value-cdr-ref)
(define parser-context-cons     ACTUAL:parser-context-cons)
(define parser-context-car      ACTUAL:parser-context-car)
(define parser-context-cdr      ACTUAL:parser-context-cdr)
(define parser-context-cdr-ref  ACTUAL:parser-context-cdr-ref)
(define make-parser-state       ACTUAL:make-parser-state)
(define parser-state-item-set   ACTUAL:parser-state-item-set)
(define parser-start-state      ACTUAL:parser-start-state)
(define parser-action           ACTUAL:parser-action)
(define lr1-parse               ACTUAL:lr1-parse)

;;; lr1-debug.scm ends here
